-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Wastelands:StartActivity()
	print("START! -- Wastelands:StartActivity()!");
	local LAUNCH_SECRET = false;
	
	-- Reload default scene if relaunched from secret level
	if SceneMan.SceneHeight ~= 1536 then
		SceneMan:LoadScene("Wastelands" , true)
	end
	
	self.IsInitialized = false;
	
	if LAUNCH_SECRET then
		dofile("Missions - Earth.rte/Scenes/Scripts/WastelandsHell.lua")
		SceneMan:LoadScene("Secret" , true)
		return
	end
	
	-- Music related variables
	self.MusicTimer = Timer()
	self.AgressiveLength = 10000
	self.IsAgressive = false
	self.IsStarted = false
	self.MusicCalm = "Out Of Sight (calm).ogg"
	self.MusicAgressive = "Hopelessness (agressive).ogg"
	self.MusicModule = "Missions - Earth Music.rte"
	self.Health = {}
	
	for i = 1, 5 do
		self.Health[i] = 0;
	end

	self:StartMusic(self.MusicModule, self.MusicCalm)
	-- End of music related variables

	-- Difficulty related variables
	if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
		self.PositiveIndex = 1.5;
		self.NegativeIndex = 0.5;
	elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
		self.PositiveIndex = 1.25;
		self.NegativeIndex = 0.75;
	elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
		self.PositiveIndex = 1.0;
		self.NegativeIndex = 1.0;
	elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
		self.PositiveIndex = 0.85;
		self.NegativeIndex = 1.15;
	elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
		self.PositiveIndex = 0.70;
		self.NegativeIndex = 1.3;
	elseif self.Difficulty <= GameActivity.MAXDIFFICULTY then
		self.PositiveIndex = 0.55;
		self.NegativeIndex = 1.45;
	end
	
	self.Wastelands = SceneMan.Scene:GetArea("Wastelands");
	self.EnemyLZ1 = SceneMan.Scene:GetArea("EnemyLZ-1");
	self.EnemyLZ2 = SceneMan.Scene:GetArea("EnemyLZ-2");

	self.Bunker1 = SceneMan.Scene:GetArea("Bunker-1");
	self.Bunker2 = SceneMan.Scene:GetArea("Bunker-2");
	
	self.braindead = {};
	self.AssignedBrains = {};

	self.MOIDLimit = 200;

	self.PlayerGold = 0;

   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	self.ZealotGuns = {"Zealot Medium Gun" , "Zealot Heavy Gun" , "Zealot Long Gun"}
	self.ZealotActors = {"Xiin" , "Plox"}
	
	self.LastPos = nil;

	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.SpawnTimer = Timer();
	self.SpawnTimer:Reset();
	
	self.CannonTimer = Timer();
	self.CannonTimer:Reset()
	self.CannonInterval = 10000 * self.PositiveIndex;
	self.ChargeInterval = self.CannonInterval - 2000;
	
	self.EvacSpawned = false;
	self.EvacDestroyed = false;
	self.AssassinsSpawned = false;
	
	-- Time between spawns
	self.SpawnInterval = 12000 * self.PositiveIndex;
	
	-- Time before evac ship arrives
	self.EvacInterval = 150000 * self.NegativeIndex

	-- Time before secret level triggered
	self.SecretInterval = 280000 * self.NegativeIndex
	self.IsRaining = false;	

	
	self.FirstZealotSighted = false;
	
	--Spawn Our Heroes
	self.Doom = CreateAHuman("Brain Doom" , "UniTec.rte");
	self.Doom.Team = self.PlayerTeam;
	--self.Doom:AddInventoryItem(CreateHDFirearm("Shotgun" , "UniTec.rte"));
	--self.Doom:AddInventoryItem(CreateHDFirearm("Chaingun" , "UniTec.rte"));
	self.Doom:AddInventoryItem(CreateHDFirearm("Rocket Launcher" , "UniTec.rte"));
	self.Doom.AIMode = Actor.AIMODE_SENTRY;
	self.Doom.Pos = SceneMan.Scene:GetArea("Doom"):GetCenterPoint();
	MovableMan:AddActor(self.Doom);

	self.Crash = CreateAHuman("Brain Crash" , "UniTec.rte");
	self.Crash.Team = self.PlayerTeam;
	self.Crash:AddInventoryItem(CreateHDFirearm("Plasmagun" , "UniTec.rte"));
	self.Crash:AddInventoryItem(CreateHDFirearm("Rocket Launcher" , "UniTec.rte"));
	self.Crash.AIMode = Actor.AIMODE_SENTRY;
	self.Crash.Pos = SceneMan.Scene:GetArea("Crash"):GetCenterPoint();
	MovableMan:AddActor(self.Crash);

	self.Phobos = CreateAHuman("Brain Phobos" , "UniTec.rte");
	self.Phobos.Team = self.PlayerTeam;
	--self.Phobos:AddInventoryItem(CreateHDFirearm("Shotgun" , "UniTec.rte"));
	self.Phobos:AddInventoryItem(CreateHDFirearm("BFG 9000" , "UniTec.rte"));
	self.Phobos.AIMode = Actor.AIMODE_SENTRY;
	self.Phobos.Pos = SceneMan.Scene:GetArea("Phobos"):GetCenterPoint();
	MovableMan:AddActor(self.Phobos);--]]--
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function Wastelands:PauseActivity(pause)
    print("PAUSE! -- Wastelands:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function Wastelands:EndActivity()
    print("END! -- Wastelands:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:SpawnZealots()
	print ("SpawnZealots")
	local ship;
	local actors;
	
	if MovableMan:GetMOIDCount() <= self.MOIDLimit then
		if self.SpawnInterval > 5000 then
			self.SpawnInterval = self.SpawnInterval - 500;
		end
	
		if not self.FirstZealotSighted or math.random() < 0.85 then
			ship = CreateACDropShip("Zealot Saucer" , "UniTec.rte")
			actors = 2;
		else
			ship = CreateACRocket("Zealot Ship" , "UniTec.rte")
			actors = 1;
		end
		
		ship.Team = self.CPUTeam;
		--ship.Pos = Vector(math.random(SceneMan.Scene.Width) , -10);
		
		-- Select enemy LZ
		local bunker1empty = true
		local bunker2empty = true
		
		for actor in MovableMan.Actors do
			if actor.Team == self.PlayerTeam then
				if self.Bunker1:IsInside(actor.Pos) then
					bunker1empty = false
				end
			
				if self.Bunker2:IsInside(actor.Pos) then
					bunker2empty = false
				end
			end
		end
		
		if (bunker1empty and bunker2empty) or (not bunker1empty and not bunker2empty) then
			if math.random() < 0.5 then
				ship.Pos = Vector(self.EnemyLZ1:GetRandomPoint().X , -10);
			else
				ship.Pos = Vector(self.EnemyLZ2:GetRandomPoint().X , -10);
			end
		else
			if not bunker1empty then
				ship.Pos = Vector(self.EnemyLZ1:GetRandomPoint().X , -10);
			end
		
			if not bunker2empty then
				ship.Pos = Vector(self.EnemyLZ2:GetRandomPoint().X , -10);
			end
		end
		
		ship:SetControllerMode(Controller.CIM_AI, -1);

		for i = 1, actors do
			actor = CreateAHuman(self.ZealotActors[math.random(#self.ZealotActors)] , "UniTec.rte");
			actor:AddInventoryItem(CreateHDFirearm(self.ZealotGuns[math.random(#self.ZealotGuns)] , "UniTec.rte"))
			actor:AddInventoryItem(CreateHDFirearm("Zealot Small Gun" , "UniTec.rte"))
			if math.random() < 0.35 then
				actor:AddInventoryItem(CreateHDFirearm("Zealot Digger" , "UniTec.rte"))
			end
			
			actor.Team = self.CPUTeam;
			--if math.random() < 0.85 then
				actor.AIMode = Actor.AIMODE_BRAINHUNT;
			--else
			--	actor.AIMode = Actor.AIMODE_SENTRY;
			--end
			ship:AddInventoryItem(actor);
		end
		
		MovableMan:AddActor(ship);
		
		if not self.FirstZealotSighted then
			self.InvisibleActor = CreateActor("Brain Case" , "Base.rte");
			self.InvisibleActor.Team = Activity.TEAM_1;
			self.InvisibleActor.Pos = Vector(ship.Pos.X, ship.Pos.Y);
			self.InvisibleActor.Scale = 0;
			self.InvisibleActor.GetsHitByMOs = false;
			self.InvisibleActor.HitsMOs = false;
			MovableMan:AddActor(self.InvisibleActor);
		end
	end
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:Dist(pos1 , pos2)
	return SceneMan:ShortestDistance(pos1, pos2, true).Magnitude;
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:NearestHero(pos)
	local dist = 10000;
	local nearest = nil;
	local d = 15000;
	
	if MovableMan:IsActor(self.Doom) then
		d = self:Dist(pos, self.Doom.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Doom;
		end
	end
	
	if MovableMan:IsActor(self.Crash) then
		d = self:Dist(pos, self.Crash.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Crash;
		end
	end

	if MovableMan:IsActor(self.Phobos) then
		d = self:Dist(pos, self.Phobos.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Phobos;
		end
	end
	
	return nearest;
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:ZealotControl()
	for actor in MovableMan.Actors do
		if actor.Team == self.CPUTeam then
			if not self.FirstZealotSighted and actor.PresetName == "Zealot Saucer" then
				for player = 0, self.PlayerCount - 1 do
					if MovableMan:IsActor(self.InvisibleActor) then
						self.InvisibleActor.Pos = Vector(actor.Pos.X, actor.Pos.Y);
						self.InvisibleActor.Team = -1;
						self:SwitchToActor(self.InvisibleActor, player, self.PlayerTeam);
						
						SceneMan:SetScroll(self.InvisibleActor.Pos, player);
						self:SetObservationTarget(self.InvisibleActor.Pos, player);
					end
				end
			end
			
			if actor.ClassName == "AHuman" then
				-- Asign actor for heroehunt and reassign already assigned from time to time
				if actor.AIMode == Actor.AIMODE_SENTRY or (actor.AIMode == Actor.AIMODE_GOTO and math.random() < 0.001) then
					local nearest;
					nearest = self:NearestHero(actor.Pos);
					
					if nearest ~= nil and MovableMan:IsActor(nearest) then
						if actor.AIMode == Actor.AIMODE_GOTO then
							-- nearest:FlashWhite(50) -- Debug
						end
						
						actor.AIMode = Actor.AIMODE_GOTO;
						actor:ClearAIWaypoints();
						actor:AddAISceneWaypoint(Vector(nearest.Pos.X , nearest.Pos.Y));
					end
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Wastelands:UpdateActivity()
	if LAUNCH_SECRET then
		return
	end

	self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);

	self:ProcessMusic(self.Doom, self.Crash, self.Phobos, nil, nil)
	
   	self:ClearObjectivePoints();

	if not self.GenericTimer:IsPastSimMS(3000) then
		for player = 0, self.PlayerCount - 1 do
			FrameMan:SetScreenText("Meanwhile...", player, 0, 1000, true);
		end
	end

	-- Show Zealot warning
	if self.GenericTimer:IsPastSimMS(5000) and not self.GenericTimer:IsPastSimMS(10000) then
		for player = 0, self.PlayerCount - 1 do
			FrameMan:ClearScreenText(player);
			FrameMan:SetScreenText("<Warning: Unknown entity signature detected. Defend.>", player, 250, 10000, true);
		end
	end
	
	if self.GenericTimer:IsPastSimMS(5000) then
		if self.SpawnTimer:IsPastSimMS(self.SpawnInterval) then
			self.SpawnTimer:Reset();
			self:SpawnZealots();
		end
	end
	
	-- Show first saucer arrival
	if not self.FirstZealotSighted then
		if self.GenericTimer:IsPastSimMS(5000 + self.SpawnInterval + 1500) then
			self.FirstZealotSighted = true;
			
			--Remove Invisible actor used to track saucer
			if MovableMan:IsActor(self.InvisibleActor) then
				self.InvisibleActor.Health = 0;
				self.InvisibleActor.Lifetime = 1;
				MovableMan:RemoveActor(self.InvisibleActor)
				self.InvisibleActor = nil;
			end
		end
	end
	
	
	if self.FirstZealotSighted then
		-- Show evac message
		if self.GenericTimer:IsPastSimMS(45000) and not self.GenericTimer:IsPastSimMS(50000) then
			for player = 0, self.PlayerCount - 1 do
				FrameMan:ClearScreenText(player);
				FrameMan:SetScreenText("More enemies incoming we'll have to evac you ASAP. Wait for evac ship.", player, 0, 10000, true);
			end
		end
		
		-- Control orbital cannon
		if self.GenericTimer:IsPastSimMS(12000) then
			-- Targeting
			if self.CannonTimer:IsPastSimMS(self.CannonInterval - self.ChargeInterval) then
				-- Get target
				if self.CannonTarget == nil then
					for actor in MovableMan.Actors do
						if actor.Team == self.PlayerTeam and actor.ClassName == "AHuman" and math.random() < 0.15 then
							self.CannonTarget = actor.Pos.X
							self.CannonTargetAlt = SceneMan:MovePointToGround(Vector(self.CannonTarget, 0) , 20 , 3).Y;
							self.LastParticle = self.CannonTimer.ElapsedSimTimeMS
						end
					end
				else
				-- Show targeting effects
					if self.CannonTimer.ElapsedSimTimeMS - self.LastParticle > 35 then
						local p = CreateMOSParticle("Engine Glow", "Missions - Earth.rte")
						p.Pos = Vector(self.CannonTarget -50 + math.random(100), self.CannonTargetAlt);
						p.Vel = Vector(0,-(4 + math.random(5)));
						MovableMan:AddParticle(p)
						self.LastParticle = self.CannonTimer.ElapsedSimTimeMS
					end
				end
				--self:AddObjectivePoint("DANGER", Vector(self.CannonTarget, self.CannonTargetAlt) , self.PlayerTeam, GameActivity.ARROWDOWN)
			end
		
			-- Shooting
			if self.CannonTimer:IsPastSimMS(self.CannonInterval) then
				if self.CannonTarget ~= nil then
					for i = 1, 10 do
						local p = CreateMOSParticle("Green Glow Big", "Missions - Earth.rte")
						p.Pos = Vector(self.CannonTarget, 0 - i * 20);
						p.Vel = Vector(0,250);
						MovableMan:AddParticle(p)
					end

					for i = 1, 6 do
						for j = 0, 1 do
							local expl = CreateTDExplosive("TDExplosive Zealot Heavy Gun", "UniTec.rte")
							expl.Pos = Vector(self.CannonTarget - 5 + (j * 10), 0 - i * 20);
							expl.Vel = Vector(0,250);
							expl.Mass = 5000;
							MovableMan:AddParticle(expl)
						end
					end

					
					--[[for i = 1, 8 do
						local expl = CreateTDExplosive("TDExplosive Zealot Heavy Gun", "UniTec.rte")
						expl.Pos = Vector(self.CannonTarget - 20 + (i * 5), 0);
						expl.Vel = Vector(0,250);
						expl.Mass = 5000;
						MovableMan:AddParticle(expl)
					end]]--
					
					self.CannonTimer:Reset();
					self.CannonTarget = nil
				end
			end
		else
			self.CannonTimer:Reset();
		end
	
		-- Process evacuation
		if self.GenericTimer:IsPastSimMS(self.EvacInterval) then
			if not self.EvacSpawned then
				-- Spawn evac ship
				self.EvacSpawned = true;
				self.EvacShip = CreateACDropShip("UniTec Mammoth" , "Unitec.rte");
				self.EvacShip.Team = self.PlayerTeam;
				self.EvacShip.Pos = Vector(math.random(SceneMan.Scene.Width), 0);
				self.EvacShip:SetControllerMode(Controller.CIM_AI, -1);				
				MovableMan:AddActor(self.EvacShip);
			else
				-- Control evac ship
				if MovableMan:IsActor(self.EvacShip) then
					--Make sure the ship doesn't try to leave without you.
					local desiredalt = SceneMan:MovePointToGround(self.EvacShip.Pos , 20 , 3).Y;
					local alt = SceneMan:FindAltitude(self.EvacShip.Pos , 0 , 3);
					
					if alt < 100 then
						self.EvacShip:GetController():SetState(Controller.MOVE_DOWN , false);
						self.EvacShip:GetController():SetState(Controller.MOVE_UP , true);
					elseif alt > 200 then
						self.EvacShip.Vel = self.EvacShip.Vel + Vector(0 , 0.05);
						self.EvacShip:GetController():SetState(Controller.MOVE_DOWN , true);
						self.EvacShip:GetController():SetState(Controller.MOVE_UP , false);
					end

					--Add an objective point if Jess isn't in the ship
					self:AddObjectivePoint("Evacuate", self.EvacShip.AboveHUDPos , self.PlayerTeam, GameActivity.ARROWUP)
					
					-- Spawn killer saucers
					if not self.AssassinsSpawned then
						if self.EvacShip:HasObject("Crash") or self.EvacShip:HasObject("Doom") or self.EvacShip:HasObject("Phobos") then
							self.AssassinsSpawned = true;
							
							local ship = CreateACDropShip("Zealot Saucer" , "UniTec.rte")
							ship.Team = self.CPUTeam;
							ship.Pos = Vector(self.EvacShip.Pos.X - 50 , 25);
							ship:SetControllerMode(Controller.CIM_AI, -1);
							MovableMan:AddActor(ship)
							
							local ship = CreateACDropShip("Zealot Saucer" , "UniTec.rte")
							ship.Team = self.CPUTeam;
							ship.Pos = Vector(self.EvacShip.Pos.X + 50 , 25);
							ship:SetControllerMode(Controller.CIM_AI, -1);
							MovableMan:AddActor(ship)
							
							local ship = CreateACDropShip("Zealot Saucer" , "UniTec.rte")
							ship.Team = self.CPUTeam;
							ship.Pos = Vector(self.EvacShip.Pos.X , -25);
							ship:SetControllerMode(Controller.CIM_AI, -1);
							MovableMan:AddActor(ship)
						end
					end
				else
					self.EvacShip = nil;
					-- Show last message
					if not self.EvacDestroyed then
						print ("Evac destroyed")
						self.EvacDestroyed = true;
					end
					
					if self.EvacDestroyed then
						for player = 0, self.PlayerCount - 1 do
							FrameMan:ClearScreenText(player);
							FrameMan:SetScreenText("LET IT BE YOUR FINEST HOUR MEN. GOODBYE.", player, 0, 20000, true);
						end
					end
				end
			end
		end

		-- Start blood rain to indicate secret activation
		if self.GenericTimer:IsPastSimMS(self.SecretInterval - 15000) then
			if not self.IsRaining then
				self:StartBloodRain()
				self.IsRaining = true;
			end
		end
		
		-- Launch secret level
		if self.GenericTimer:IsPastSimMS(self.SecretInterval) then
			dofile("Missions - Earth.rte/Scenes/Scripts/WastelandsHell.lua")
			SceneMan:LoadScene("Secret" , true)
			return
		end
	end
	
	if self.ActivityState ~= Activity.OVER then
		self:CheckWinner();
		self:ZealotControl();
	end

	-- Just for debug but still funny if enabled =)
	--for actor in MovableMan.Actors do
	--	actor.Health = actor.Health - math.random();
	--end
	
	--self:DoBrainSelection()
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Wastelands:StartBloodRain()
	local rainrunner = CreateMOPixel("Particle Blood Rain Manager","Missions - Earth.rte");
	rainrunner.Mass = 10000--duration;
	rainrunner.Sharpness = 100000--ppm;
	rainrunner.Pos = Vector(0,40);--speed
	MovableMan:AddParticle(rainrunner);
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Wastelands:DoBrainSelection()
	if not (self.ActivityState == Activity.OVER) then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);

			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) then
				self:SetPlayerBrain(nil, player);
				local newBrain = self.Doom;
				
				if newBrain == nil then
					newBrain = self.Phobos;
				end
				
				if newBrain == nil then
					newBrain = self.Crash;
				end
				
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self.AssignedBrains[player] = newBrain;
					self:SwitchToActor(newBrain, player, team);
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Check winning conditions
-----------------------------------------------------------------------------------------
function Wastelands:CheckWinner()
	if not MovableMan:IsActor(self.Doom) then
		self.Doom = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Doom", self.Doom.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Doom.Health <= 0 then
			self.LastPos = Vector(self.Doom.Pos.X , self.Doom.Pos.Y)
			self.Doom = nil;
		
			for actor in MovableMan.Actors do
				if actor.Team == self.CPUTeam then
					actor.AIMode = Actor.AIMODE_BRAINHUNT
				end
			end
		end
	end
	
	if not MovableMan:IsActor(self.Crash) then
		self.Crash = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Crash", self.Crash.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Crash.Health <= 0 then
			self.LastPos = Vector(self.Crash.Pos.X , self.Crash.Pos.Y)
			self.Crash = nil;
		
			for actor in MovableMan.Actors do
				if actor.Team == self.CPUTeam then
					actor.AIMode = Actor.AIMODE_BRAINHUNT
				end
			end
		end
	end

	if not MovableMan:IsActor(self.Phobos) then
		self.Phobos = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Phobos", self.Phobos.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Phobos.Health <= 0 then
			self.LastPos = Vector(self.Phobos.Pos.X , self.Phobos.Pos.Y)
			self.Phobos = nil;
		
			for actor in MovableMan.Actors do
				if actor.Team == self.CPUTeam then
					actor.AIMode = Actor.AIMODE_BRAINHUNT
				end
			end
		end
	end

	if self.ActivityState ~= Activity.OVER then
		if self.Doom == nil and self.Crash == nil and self.Phobos == nil then
		    -- Put fake brain just before the end of activity or the game will crash due to "brainless" gameplay
			for player = 0, self.PlayerCount - 1 do
				self.InvisibleActor = CreateActor("Brain Case" , "Base.rte");
				self.InvisibleActor.Team = Activity.TEAM_1;
				self.InvisibleActor.Pos = self.LastPos;
				self.InvisibleActor.Scale = 0;
				self.InvisibleActor.GetsHitByMOs = false;
				self.InvisibleActor.HitsMOs = false;
				MovableMan:AddActor(self.InvisibleActor);
				self:SetPlayerBrain(self.InvisibleActor , player);
				self:SwitchToActor(self.InvisibleActor, player, self.PlayerTeam);
			end

			if self.EvacDestroyed then
				self.WinnerTeam = self.PlayerTeam;
			else
				self.WinnerTeam = self.CPUTeam;
			end
			ActivityMan:EndActivity();
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
				FrameMan:ClearScreenText(player);
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Triggers agressive music if some of critical actors was wounded
-----------------------------------------------------------------------------------------
function Wastelands:ProcessMusic(a1,a2,a3,a4,a5)
	local actors = {}
	
	actors[1] = a1;
	actors[2] = a2;
	actors[3] = a3;
	actors[4] = a4;
	actors[5] = a5;
	
	for i = 1, 5 do
		if MovableMan:IsActor(actors[i]) then
			if actors[i].Health < self.Health[i] then
				if not self.IsAgressive then
					self.IsAgressive = true;
					self:StartMusic(self.MusicModule, self.MusicAgressive)
				end
				self.MusicTimer:Reset();
				self.Health[i] = actors[i].Health
				break;
			end
			self.Health[i] = actors[i].Health
		end
	end

	if self.IsAgressive then
		if self.MusicTimer:IsPastSimMS(self.AgressiveLength) then
			self.IsAgressive = false;
			self:StartMusic(self.MusicModule, self.MusicCalm)
		end
	end
end
-----------------------------------------------------------------------------------------
-- Starts music from file if it's installed
-----------------------------------------------------------------------------------------
function Wastelands:StartMusic(modulename , musicfile)
	local path = "./"..modulename.."/"..musicfile

	if self:IsFilePathExists(path) then
		AudioMan:ClearMusicQueue();
		AudioMan:PlayMusic(path, -1, -1);
	else
		print ("Can't find music: "..path);
	end	
end
-----------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------
function Wastelands:IsFilePathExists(path)
	io = require("io");
	
	local file = io.open(path , "r");
	
	if file == nil then
		return false;
	end
	
	file:close();
	return true;
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































