-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function Wastelands:StartActivity()
	print("START! -- Wastelands Hell:StartActivity()!");

	if self.IsInitialized == nil then
		self.IsInitialized = false
	end

	if self.IsInitialized then 
		return
	end
	
	-- Music related variables
	self.MusicTimer = Timer()
	self.AgressiveLength = 10000
	self.IsAgressive = false
	self.IsStarted = false
	self.MusicCalm = "Musorgskiy (calm).ogg"
	self.MusicAgressive = "ComeToDaddy (agressive).ogg"
	self.MusicModule = "Missions - Earth Music.rte"
	self.Health = {}

	self.ToScream = true;
	self.ToFire = true;
	
	self.Demon = nil
	
	for i = 1, 5 do
		self.Health[i] = 0;
	end

	self:StartMusic(self.MusicModule, self.MusicCalm)
	-- End of music related variables

	-- Difficulty related variables
	if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
		self.PositiveIndex = 1.5;
		self.NegativeIndex = 0.5;
	elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
		self.PositiveIndex = 1.25;
		self.NegativeIndex = 0.75;
	elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
		self.PositiveIndex = 1.0;
		self.NegativeIndex = 1.0;
	elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
		self.PositiveIndex = 0.85;
		self.NegativeIndex = 1.15;
	elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
		self.PositiveIndex = 0.70;
		self.NegativeIndex = 1.3;
	elseif self.Difficulty <= GameActivity.MAXDIFFICULTY then
		self.PositiveIndex = 0.55;
		self.NegativeIndex = 1.45;
	end
	
	self.braindead = {};
	self.AssignedBrains = {};

	for player = 0, self.PlayerCount - 1 do
		self.braindead[player] = true;
	end
	
	self.MOIDLimit = 200;

	self.PlayerGold = 0;

   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;
	
	self.LastPos = nil;
	self.LastDemonPos = nil;

	self.GenericTimer = Timer();
	self.GenericTimer:Reset();

	self.SpawnTimer = Timer();
	self.SpawnTimer:Reset();
	self.SpawnInterval = 10000 * self.PositiveIndex;
	
	self.WeaponTimer = Timer();
	self.WeaponTimer:Reset();
	self.WeaponInterval = 15000 * self.NegativeIndex;
	
	self.AttackTimer = Timer();
	self.AttackTimer:Reset();
	self.AttackInterval = 2500 * self.NegativeIndex;
	
	self.DoomWeapons = {"BFG 9000", "Chaingun", "Rocket Launcher", "Plasmagun"}
	
	self.FirstZealotSighted = false;
	
	--Spawn Our Heroes
	self.Doom = CreateAHuman("Brain Doom" , "UniTec.rte");
	self.Doom.Team = self.PlayerTeam;
	self.Doom.AIMode = Actor.AIMODE_SENTRY;
	self.Doom.Pos = SceneMan.Scene:GetArea("Doom"):GetCenterPoint();
	MovableMan:AddActor(self.Doom);
	
	self.Crash = CreateAHuman("Brain Crash" , "UniTec.rte");
	self.Crash.Team = self.PlayerTeam;
	self.Crash.AIMode = Actor.AIMODE_SENTRY;
	self.Crash.Pos = SceneMan.Scene:GetArea("Crash"):GetCenterPoint();
	MovableMan:AddActor(self.Crash);

	self.Phobos = CreateAHuman("Brain Phobos" , "UniTec.rte");
	self.Phobos.Team = self.PlayerTeam;
	self.Phobos.AIMode = Actor.AIMODE_SENTRY;
	self.Phobos.Pos = SceneMan.Scene:GetArea("Phobos"):GetCenterPoint();
	MovableMan:AddActor(self.Phobos);--]]--

	local fx = CreateAEmitter("Cacodemon Roar");
	fx.Pos = self.Doom.Pos;
	MovableMan:AddParticle(fx);	
	
	-- Spawn train
	--self.Demon = CreateACRocket("Ronin Station Bullet Cart One" , "Missions - Earth.rte");
	self.Demon = CreateACRocket("Cacodemon" , "Missions - Earth.rte");
	self.Demon.Pos = Vector(2100, 100);
	self.Demon.Team = self.CPUTeam;
	self.Demon.AIMode = Actor.AIMODE_STAY;
	MovableMan:AddActor(self.Demon);
	--self.Demon:FlashWhite(750);
	
	self:StartBloodRain()
	
	self.IsInitialized = true	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function Wastelands:PauseActivity(pause)
    print("PAUSE! -- Wastelands:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function Wastelands:EndActivity()
    print("END! -- Wastelands:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:Dist(pos1 , pos2)
	return SceneMan:ShortestDistance(pos1, pos2, true).Magnitude;
end
-----------------------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------------------
function Wastelands:NearestHero(pos)
	local dist = 10000;
	local nearest = nil;
	local d = 15000;
	
	if MovableMan:IsActor(self.Doom) then
		d = self:Dist(pos, self.Doom.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Doom;
		end
	end
	
	if MovableMan:IsActor(self.Crash) then
		d = self:Dist(pos, self.Crash.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Crash;
		end
	end

	if MovableMan:IsActor(self.Phobos) then
		d = self:Dist(pos, self.Phobos.Pos);
	
		if d < dist then
			dist = d;
			nearest = self.Phobos;
		end
	end
	
	return nearest;
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function Wastelands:UpdateActivity()
	-- Just check for intialization flags in update loop to avoid unnecessary function calls during all the mission
	if self.IsInitialized == nil then
		self.IsInitialized = false
	end
	
	if not self.IsInitialized then
		--Init mission if we're still not
		self:StartActivity()
	end

	self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
	self:ProcessMusic(self.Doom, self.Crash, self.Phobos, nil, nil)
   	self:ClearObjectivePoints();
	
	if self.ActivityState ~= Activity.OVER then
		self:CheckWinner();
	end

	if self.GenericTimer:IsPastSimMS(2000) and not self.GenericTimer:IsPastSimMS(8000) then
		if MovableMan:IsActor(self.Demon) then
			for player = 0, self.PlayerCount - 1 do
				-- Thanks again Bad Boy ;)
				SceneMan:SetScrollTarget(self.Demon.Pos, 0.04, false, player);
			end
			self:AddObjectivePoint("Kill", self.Demon.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end
	end
	
	-- Highlight items
	--for item in MovableMan.Items do
	--	self:AddObjectivePoint(item.PresetName, item.Pos + Vector(0,-20), self.PlayerTeam, GameActivity.ARROWDOWN);
	--end

	-- Drop weapons
	if self.WeaponTimer:IsPastSimMS(self.WeaponInterval) then
		local w = CreateHDFirearm(self.DoomWeapons[math.random(#self.DoomWeapons)], "UniTec.rte")
		w.Pos = Vector(math.random(SceneMan.Scene.Width), 0)
		w.Vel = Vector(0, 50)
		MovableMan:AddItem(w)
		self.WeaponTimer:Reset();
	end
	
	-- Drop skeletons
	if self.SpawnTimer:IsPastSimMS(self.SpawnInterval) then
		if MovableMan:GetMOIDCount() <= self.MOIDLimit then
			local a = CreateAHuman("Skeleton", "Undead.rte")
			a.Team = self.CPUTeam
			a:AddInventoryItem(CreateHDFirearm("Shotgun", "UniTec.rte"))
			a.Pos = Vector(math.random(SceneMan.Scene.Width), 0)
			MovableMan:AddActor(a)
			self.SpawnTimer:Reset();
		end
	end
	
	-- Control demon
	if MovableMan:IsActor(self.Demon) then
		self.Demon.RotAngle = 0;
	
		-- Move
		local hero = self:NearestHero(self.Demon.Pos) 
		local fireballaim = nil
		local fireballpos = nil
		
		if MovableMan:IsActor(hero) then
			local pos = hero.Pos
			local dist = self:Dist(self.Demon.Pos, pos)
			
			if dist > 150 then
				local inc = ((pos.X - self.Demon.Pos.X) / (dist / 2))
				if math.abs(self.Demon.Vel.X + inc) < 10 then
					self.Demon.Vel.X = self.Demon.Vel.X + inc
				end
				
				if self.Demon.Vel.X > 9.99 then
					self.Demon.Vel.X = 9.99
				end

				if self.Demon.Vel.X < -9.99 then
					self.Demon.Vel.X = -9.99
				end
				
			end
			
			fireballpos = Vector((pos.X - self.Demon.Pos.X) / dist * 60, (pos.Y - self.Demon.Pos.Y) / dist * 60)
			fireballaim = Vector((pos.X - self.Demon.Pos.X) / dist * 40, (pos.Y - self.Demon.Pos.Y) / dist * 40)
		end
		self.LastDemonPos = self.Demon.Pos
		
		-- Attack
		if self.AttackTimer:IsPastSimMS(self.AttackInterval) then
			if fireballaim ~= nil then
				local frame = math.floor((self.AttackTimer.ElapsedSimTimeMS - self.AttackInterval) / 150)
				
				if frame == 1 and self.ToScream then
					local fx = CreateAEmitter("Cacodemon Roar");
					fx.Pos = self.LastDemonPos;
					MovableMan:AddParticle(fx);	
					self.ToScream = false;
				end

				if frame == 3 and self.ToFire then
					--self.Demon:FlashWhite(500)
					for i = 1, 2 do
						local fx = CreateAEmitter("Destroyer Cannon Shot");
						fx.Pos = self.LastDemonPos + fireballpos;
						fx.Vel = fireballaim;
						MovableMan:AddParticle(fx);	
					end
					for i = 1, 6 do
						local fx = CreateAEmitter("UniTec Ground Flames");
						fx.Pos = self.LastDemonPos + fireballpos;
						fx.Vel = fireballaim;
						MovableMan:AddParticle(fx);	

						local fx = CreateAEmitter("Explosion Trail 1");
						fx.Pos = self.LastDemonPos + fireballpos;
						fx.Vel = fireballaim;
						MovableMan:AddParticle(fx);	
					end
					self.ToFire = false;
				end
				
				if frame > 6 then 
					frame = 6;
				end
				
				self.Demon.Frame = frame
				
				if self.AttackTimer:IsPastSimMS(self.AttackInterval + 150 * 6) then
					self.Demon.Frame = 0
					self.AttackTimer:Reset();
					self.ToScream = true;
					self.ToFire = true;
				end
			end
		end
	else
		self.Demon = nil
		-- Demon is dead - victory
		if self.ActivityState ~= Activity.OVER then
			self.WinnerTeam = self.PlayerTeam;
			ActivityMan:EndActivity();
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
				FrameMan:ClearScreenText(player);
			end
		end
	end
	
	self:DoBrainSelection()
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function Wastelands:DoBrainSelection()
	if self.ActivityState ~= Activity.OVER then
		if self.IsInitialized then
			for player = 0, self.PlayerCount - 1 do
				local team = self:GetTeamOfPlayer(player);
				local brain = self:GetPlayerBrain(player);
				if not MovableMan:IsActor(brain) then
					self:SetPlayerBrain(nil, player);
					local newBrain = MovableMan:GetUnassignedBrain(team);
					if newBrain and self.braindead[player] then
						self:SetPlayerBrain(newBrain, player);
						self.AssignedBrains[player] = newBrain;
						self:SwitchToActor(newBrain, player, team);
					else
						if self.ActivityState ~= Activity.OVER then
							self.WinnerTeam = self.CPUTeam;
							ActivityMan:EndActivity();
							for player = 0, self.PlayerCount - 1 do
								self.braindead[player] = true;
								FrameMan:ClearScreenText(player);
							end
						end
					end
				else
					self:SetObservationTarget(brain.Pos, player)
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Check winning conditions
-----------------------------------------------------------------------------------------
function Wastelands:CheckWinner()
	if not MovableMan:IsActor(self.Doom) then
		self.Doom = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Doom", self.Doom.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Doom.Health <= 0 then
			self.LastPos = Vector(self.Doom.Pos.X , self.Doom.Pos.Y)
			self.Doom = nil;
		else
			self.Doom.Health = 100
		end
	end
	
	if not MovableMan:IsActor(self.Crash) then
		self.Crash = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Crash", self.Crash.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Crash.Health <= 0 then
			self.LastPos = Vector(self.Crash.Pos.X , self.Crash.Pos.Y)
			self.Crash = nil;
		else
			self.Crash.Health = 100
		end
	end

	if not MovableMan:IsActor(self.Phobos) then
		self.Phobos = nil;
	else
		if not self.GenericTimer:IsPastSimMS(5000) then
			self:AddObjectivePoint("Phobos", self.Phobos.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		end

		if self.Phobos.Health <= 0 then
			self.LastPos = Vector(self.Phobos.Pos.X , self.Phobos.Pos.Y)
			self.Phobos = nil;
		else
			self.Phobos.Health = 100
		end
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function Wastelands:StartBloodRain()
	local rainrunner = CreateMOPixel("Particle Blood Rain Manager","Missions - Earth.rte");
	rainrunner.Mass = 10000--duration;
	rainrunner.Sharpness = 100000--ppm;
	rainrunner.Pos = Vector(0,40);--speed
	MovableMan:AddParticle(rainrunner);
end
-----------------------------------------------------------------------------------------
-- Triggers agressive music if some of critical actors was wounded
-----------------------------------------------------------------------------------------
function Wastelands:ProcessMusic(a1,a2,a3,a4,a5)
	local actors = {}
	
	actors[1] = a1;
	actors[2] = a2;
	actors[3] = a3;
	actors[4] = a4;
	actors[5] = a5;
	
	for i = 1, 5 do
		if MovableMan:IsActor(actors[i]) then
			if actors[i].Health < self.Health[i] then
				if not self.IsAgressive then
					self.IsAgressive = true;
					self:StartMusic(self.MusicModule, self.MusicAgressive)
				end
				self.MusicTimer:Reset();
				self.Health[i] = actors[i].Health
				break;
			end
			self.Health[i] = actors[i].Health
		end
	end

	if self.IsAgressive then
		if self.MusicTimer:IsPastSimMS(self.AgressiveLength) then
			self.IsAgressive = false;
			self:StartMusic(self.MusicModule, self.MusicCalm)
		end
	end
end
-----------------------------------------------------------------------------------------
-- Starts music from file if it's installed
-----------------------------------------------------------------------------------------
function Wastelands:StartMusic(modulename , musicfile)
	local path = "./"..modulename.."/"..musicfile

	if self:IsFilePathExists(path) then
		AudioMan:ClearMusicQueue();
		AudioMan:PlayMusic(path, -1, -1);
	else
		print ("Can't find music: "..path);
	end	
end
-----------------------------------------------------------------------------
-- 
-----------------------------------------------------------------------------
function Wastelands:IsFilePathExists(path)
	io = require("io");
	
	local file = io.open(path , "r");
	
	if file == nil then
		return false;
	end
	
	file:close();
	return true;
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































